/*
    magic.h

    This file contains the function declarations for the magic functions.
*/

#ifndef NCHESS_SRC_MAGICS_H
#define NCHESS_SRC_MAGICS_H

#include "core.h"
#include "types.h"
#include "config.h"


// This function finds magic numbers for sliding pieces and prints them
// to the console to copy paste them to nchess files.
void find_all_magic_numbers();

#endif