#ifndef NCHESS_CORE_PYBOARD_METHODS_H
#define NCHESS_CORE_PYBOARD_METHODS_H

#define PY_SSIZE_CLEAN_H
#include <Python.h>

extern PyMethodDef pyboard_methods[];

#endif // NCHESS_CORE_PYBOARD_METHODS_H