/*
    types.h

    This file contains type definitions used throughout the program.
*/

#ifndef NCHESS_SRC_TYPES_H
#define NCHESS_SRC_TYPES_H

typedef unsigned long long uint64;
#define NCH_UINT64_MAX 0xFFFFFFFFFFFFFFFF

typedef unsigned char uint8;
typedef unsigned short uint16;
typedef unsigned int uint32;

#endif // NCHESS_SRC_TYPES_H